<?php
/**
 * Clock Plugin: Shows realtime clock
 * 
 * @file       clock/syntax.php
 * @brief      Show a clock in wikipage
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Luis Machuca B. <luis.machuca@gulix.cl>
 * @version    1.0
 * 
 *  -----
 *  No longer in 'experimental'!
 *  -----
 *
 *  This plugin's purpose is to display a clock using both 
 *  CSS and JavaScript techniques normally available.
 *
 *   For a live test point a decent web browser to my wiki.
 *  http://informatica.temuco.udelmar.cl/~lmachuca/dokuwiki-lucho/
 *
 *  Greetings.
 *        - Luis Machuca B. a.k.a. 'ryan.chappelle'
 */


if(!defined('DW_LF')) define('DW_LF',"\n");
 
if(!defined('DOKU_INC')) 
define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) 
define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_clock extends DokuWiki_Syntax_Plugin {
 
    /**
     * return some info
     */
    function getInfo(){
        return array(
            'author' => 'Luis Machuca B.',
            'email'  => 'luis.machuca@gulix.cl',
            'date'   => '2008-12-24',
            'name'   => 'Clock Plugin',
            'desc'   => 'Shows a JavaScript-based clock',
            'url'    => 'http://wiki.splitbrain.org/plugin:clock',
        );
    }
 
    /**
     * What kind of syntax are we?
     */
    function getType(){
        return 'substition';
    }
 
    /**
     * What can we Do?
     */
    //function getAllowedTypes() { 
    //    return array(); 
    //}
    /* A/N: getAllowedTyped is intentionally non-defined. */
 
    /**
     * Where to sort in?
     */
    function getSort(){
        return 299;
    }
 
    /**
     * What's our code layout?
     */    
    function getPType(){ 
        return 'normal'; 
    }
 
    /**
     * How do we get to the lexer?
     */
    function connectTo($mode) {
         $this->Lexer->addSpecialPattern (
         '^\{\{clock\}\}$', $mode, 'plugin_clock');
 
    }
 
    /**
     * Handle the match
     */
    function handle($match, $state, $pos, &$handler){
 
        $data= array();
        // get the clock ID and JS fallback flag
        $theCS= $this->getConf('clock_style');
        $theJS= $this->getConf('nojs_fallback');
        $theWL= $this->getConf('clock_infopage');
        // if 'nojs_fallback' is set, we get the time from the server
        // if 'clock_infopage' contains a link, we convert it

        /* compose the data array */
        $data['style']   = $theCS;
        $data['text']    = $theJS ? date('H:i:s') : $theCS;
        $data['dolink']  = !empty($theWL);
        $data['target']  = !empty($theWL) ? wl($theWL) : '';
        
        /* Are we ready yet? */

        return $data;
    }  
 
    /**
     * Create output
     */
    function render($mode, &$renderer, $data) {
        /* get the data from $data */
		static $wasrendered= false;

		if ($wasrendered == true) {
			/* already rendered: nothing to do.
			 */
			return true;
 		}

        if($mode == 'xhtml'){
            $renderer->doc .= $this->_clock_createblock_html($data);
			$wasrendered= true;
            return true;
        }
        if($mode == 'odt'){
            return false; // may be implemented in the future
        }
        /* That's all about rendering that needs to be done, at the moment */
        return false;
    }

    /**
     * From this point onwards, local (helper) functions are implemented
     */

    /** 
     * @fn          dw_clock_object_ID
     * @brief       returns ID for the clock object
     *  This function sets the name used for the ID of the clock object.
     *  It is intended to return a somewhat unique name.
     *  If you want to change it, you may also want to update 
     *  the #id tags in the 'style.css' file. 
     */
    function _get_clock_object_ID () {
        return 'dw_clock_object';
        }

    /**
     * @fn          dw_clock_object_infoelem
     * @brief       Returns the tip fot the help element
     *  This function returns the content (text) associated with the 
     *  clock object's "help link". The text here is 
     *  *not parsed by DokuWiki*.
     */
    function _get_clock_object_infoelem () {
        return '(?)';
        }


    /**
     * @fn          _clock_createblock_html
     * @brief       Creates the HTML code for the clock object.
     */
    function _clock_createblock_html($data) {
        $theStyle   = $data['style'];
        $theText    = $data['text'];
        $theDoLink  = $data['dolink'];
        $theTarget  = $data['target'];

        $codetext.= '<div id="'. $this->_get_clock_object_ID(). '"' .
                    ' class="'. $theStyle. '">';
        $codetext.= '<div id="clock_face" class="face">'. $theText.
                    '</div>'. DW_LF;

        if ($theDoLink) {
          $codetext.= '<span class="info">';
          $codetext.= '<a href="'. $theTarget. '">'. 
                      $this->_get_clock_object_infoelem(). '</a>'. 
                      '</span>';
          }
        
        $codetext.= '</div>'. DW_LF;
        return $codetext;
     }

}

 
/*?> */

